-- Auther: aramsay
-- Create Date: 3/22/2009 9:10 PM
-- Status: Alpha
-- TODO:
--      (1) "rage" bar type (just a rage bar)
--      (1) "totem" bar type
--      (1) "cast" bar type
--      (1) "energy" bar type (inverted energy bar essentially, value = cost - current)
--      (1) "channel" bar type
--      (2) "proc" and/or "internal CD" bars
--      (2) "item" bar type, for Use trinkets (might not need, if you can get the spell ID)
--      (2) "special" bar type(s), for Overpower/Taste For Blood, and other special abilities
--      (2) some way to handle DK runes
--      (2) setting to remove icons per frame
--      (2) allow a way to specify "stacking" bars, like WF/SS or Wolves/WolvesCD
--      (3) debuff bar to check for buffs in same category (allow 2ndary,etc spells to be specified);
--      (3) allow multiple of the same spell to be stored
--      (3) detect spec change and load different profile/bars
--      (3) vertical marking (like GCD, or energy abilities)
--      (3) ticks
--      (3) export/import for sharing (can currently use macros as a work-around import)


AweInspired = LibStub("AceAddon-3.0"):NewAddon("AweInspired", "AceConsole-3.0", "AceEvent-3.0")

local media = LibStub:GetLibrary("LibSharedMedia-3.0")
local frameOffset = 0;
local stophide = false;
local bolMoveable = false;
local threshold = 1.5;
local numReset = 30;
local curBarName = "name";
local curSpellID = 1;
local curBarType = "cooldown";
local curBarColor = {r=1, g=1, b=1, a=1};
local curLongBar = "wait";
				--"wait" holds the bar at full until it's remaining time is less than baseCD then starts
				--"scale" scales the bar to empty slower, losing the relative timing
				--"long" extends the bar beyond the width of baseCD and empties at the same rate of all bars.
				--"hybrid" runs scale while time left is > base CD, then refills and runs normal for the rest.

AweInspired.textures = {};
AweInspired.borders = {};

-----
-- Options passed to AceConfig-3.0 to enable slash commands.
-----
local options = {
	name = "AweInspired",
	handler = AweInspired,
	type = 'group',
	args = {
--		addframe = {
--			type = "input",
--			name = "addframe",
--			desc = "Add a frame",
--			set = "AddFrame",
--		},
--		startcd = {
--			type = "input",
--			name = "startcd",
--			desc = "Test start a CD",
--			set = "StartCD",
--		},
		deleteframe = {
			type = "input",
			name = "Delete Bar",
			desc = "Type in the spell ID of the bar to delete and press enter to remove it.",
			set = "DeleteFrame",
		},
		clearbars = {
			type = 'execute',
			name = "Clear Bars",
			desc = "Delete all bars",
			func = "ClearBars",
		},
		moveable = {
			type = 'toggle',
			name = "Move Bars",
			desc = "Enable bars to be dragged on the screen.",
			get = function() return bolMoveable; end,
			set = function(info, val)
					bolMoveable = val;
					AweInspired.BaseFrame:EnableMouse(bolMoveable);
					if bolMoveable then
						AweInspired.BaseFrame:SetBackdropColor(1, 1, 1, 1);
					else
						AweInspired.BaseFrame:SetBackdropColor(1, 1, 1, 0);
					end
				end,
			tristate = false,
		},
		general = {
			name = "General Options",
			type = 'group',
			args = {
				texture = {
					type = 'select',
					name = "Bar Texture",
					get = function() return AweInspired.db.profile.texture end,
					set = "SetBarTexture",
					values = AweInspired.textures,
				},
				border = {
					type = 'select',
					name = "Border Texture",
					get = function() return AweInspired.db.profile.border end,
					set = "SetBorderTexture",
					values = AweInspired.borders,
				},
				barheight = {
					type = "range",
					name = "Bar Height",
					desc = "The height of each bar.",
					min = 5,
					max = 100,
					step = 1,
					bigStep = 1,
					get = function(info) return AweInspired.db.profile.fHeight end,
					set = function(info,val) AweInspired.db.profile.fHeight = val end,
					isPercent = false,
				},
				barwidth = {
					type = "range",
					name = "Frame Width",
					desc = "Sets the width (in pixels) of bars with a cooldown equal to Base Cooldown (changing Long Bar settings may result in a wider frame).",
					min = 50,
					max = 1000,
					step = 1,
					bigStep = 1,
					get = function(info) return AweInspired.db.profile.fWidth end,
					set = function(info,val) AweInspired.db.profile.fWidth = val end,
					isPercent = false,
				},
				basecd = {
					type = "range",
					name = "Base Cooldown",
					desc = "The number in seconds of the max size bar, anything longer than this will be handled based on Long Bar settings.",
					min = 1,
					max = 60,
					step = 0.01,
					bigStep = 1,
					get = function(info) return AweInspired.db.profile.baseCD end,
					set = function(info,val) AweInspired.db.profile.baseCD = val end,
					isPercent = false,
				},
				scale = {
					type = "range",
					name = "Scale",
					desc = "UI Scale of the bars.",
					min = 0.1,
					max = 2,
					step = 0.01,
					bigStep = 0.05,
					get = function(info) return AweInspired.db.profile.scale; end,
					set = function(info,val) 
							AweInspired.BaseFrame:SetScale(val);
							AweInspired.db.profile.scale = val;
						end,
					isPercent = true,
				}
			},
		},
		newframe = {
			name = "Create Frame",
			type = 'group',
			args = {
				name = {
					type = "input",
					name = "Name",
					desc = "Name for the bar, only used internally or to delete the bar later",
					set = function(info,val) curBarName = val; end,
					get = function(info) return curBarName; end,
				},
				spellID = {
					type = "input",
					name = "Spell ID",
					desc = "Spell ID for new frame.",
					set = function(info,val) curSpellID = val; end,
					get = function(info) return curSpellID; end,
				},
				bartype = {
					type = "select",
					name = "Bar Type",
					desc = "Type of bar (cooldown, buff, debuff, gcd).",
					values = { 
						["cooldown"]="Cooldown", 
						["buff"]="Buff", 
						["debuff"]="Debuff",
						["gcd"]="GCD",
						["energy"]="Energy",
						["combo"]="Combo Points",
					},
					set = function(info,val) curBarType = val end,
					get = function(info) return curBarType end,
					style = "dropdown",
				},
				barcolor = {
					type = "color",
					name = "Bar Color",
					desc = "Color of the bar.",
					set = function(info,r,g,b,a) 
							curBarColor = {r=r, g=g, b=b, a=a}; 
						end,
					get = function(info) 
							return curBarColor.r, curBarColor.g, curBarColor.b, curBarColor.a;
						end,
					hasAlpha = true,
				},
				longbar = {
					type = "select",
					name = "Long Bar",
					desc = "When the cooldown is longer than Base Cooldown, how to treat the bar \n\n\"Wait\" - Keep the bar full until the remaining duration is less than Base Cooldown.\n\n\"Scale\" - Scale the time down to fit in the same width, the relative speed of the bars is no longer constant.\n\n\"Long\" - Extend the width of the bar to represent the entire duration/cooldown of the spell.\n\n\"Hybrid\" - Run the spell as \"Scaled\" until the remaining time is less than Base Cooldown, then refill the bar and behave like \"Wait\" for the remainder of the duration.",
					values = {
						["wait"]="Wait",
						["scale"]="Scale",
						["long"]="Long",
						["hybrid"]="Hybrid",
					},
					get = function(info) return curLongBar end,
					set = function(info,val) curLongBar = val end,
					style = "dropdown",
				},
				create = {
					type = "execute",
					name = "Create",
					desc = "Create the bar with the specified settings.",
					func = function(info)
							AweInspired:CreateBarFrame(curBarName, curSpellID, curBarType, curBarColor, true, curLongBar);
							table.insert(AweInspired.db.profile.bars, {name = curBarName, spellID = curSpellID, barType = curBarType, color = curBarColor, icon = true, longBar = curLongBar});
							curBarName = "";
							curSpellID = 1;
							curBarType = "cooldown";
							curBarColor = {1, 1, 1, 1};
						end,
				},
			},
		},
	},
};

-----
-- Default values for saved variables data, used with AceDB-3.0
-----
local defaults = {
	profile = {
		texture = "Blizzard",
		border = "None",
		fHeight = 25,
		fWidth = 300,
		baseCD = 10, --The width that the bar frames should use
		point = "CENTER",
		relativeTo = "UIParent",
		relativePoint = "CENTER",
		xOffset = 0,
		yOffset = 0,
		scale = 1,
		bars = {
			['**'] = {
				spellID = 1,
				barType = "cooldown",
				color = {r=1, g=1, b=1, a=1},
				icon = true,
				name = "name",
				longBar = "wait", --what to do if the bar needs to be longer than baseCD,
						--"wait" holds the bar at full until it's remaining time is less than baseCD then starts
						--"scale" scales the bar to empty slower, losing the relative timing
						--"long" extends the bar beyond the width of baseCD and empties at the same rate of all bars.
						--"hybrid" runs scale while time left is > base CD, then refills and runs normal for the rest.
			},
		},
	},
}

AweInspired.frames = {};
AweInspired.barBackdrop = {
	--bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = false, tileSize = 0, edgeSize = 12,
	insets = { left = 2, right = 2, top = 2, bottom = 2 }
};

AweInspired.frameBackdrop = {
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = false, tileSize = 0, edgeSize = 12,
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
};

AweInspired.BaseFrame = CreateFrame("Frame", "AweInspiredBase", UIParent)


-----
-- Initialization, fires when logging on or reloading UI.
-- Register the slash commands, and the bliz interface options menu using AceConfig
-- Set up the saved variables self.db wiht AceDB-3.0.
-- Create the base frame that will be the anchor for the bar frames.
-- Register shared media, i have no idea how to do this yet
-----
function AweInspired:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AweInspiredDB", defaults);
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AweInspired", options);
	media.RegisterCallback(self, "LibSharedMedia_Registered");
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AweInspired", "AweInspired");
	self:RegisterChatCommand("aweinspired", "ChatCommand");
	self:RegisterChatCommand("ais", "ChatCommand");
	self:Print("AweInspired loaded");
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig");
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig");
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig");
end

-----
-- I have no idea what this does, I copied it from ShockAndAwe to try and incorporate SharedMedia
-----
function AweInspired:LibSharedMedia_Registered()
	for k, v in pairs(media:List("statusbar")) do
		self.textures[v] = v;
	end
	for k, v in pairs(media:List("border")) do
		self.borders[v] = v
	end
end

-----
-- Fired when you enable the mod, register all event handlers here
-----
function AweInspired:OnEnable()
	self:LibSharedMedia_Registered();
	--self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");

	self:RefreshConfig();
end

-----
-- Fired when you disable the mod, unregister all event handlers here, so they don't fire while disabled
-----
function AweInspired:OnDisable()
	--self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:UnregisterEvent("PLAYER_REGEN_DISABLED");
	self:UnregisterEvent("PLAYER_REGEN_ENABLED");
end


-----
-- This function fires when exiting combat.  Disable all bars.
-----
function AweInspired:PLAYER_REGEN_ENABLED()
	for i,v in pairs(self.frames) do
		v:Hide();
		v.icon:Hide();
	end
end

-----
-- This function fires when entering combat, enable all bars.
-----
function AweInspired:PLAYER_REGEN_DISABLED()
	for i,v in pairs(self.frames) do
		--self:Print("showing index ["..i.."]");
		v:Show();
		v.icon:Show();
	end
end

-----
-- Create the base frame, this will be the anchor for the other bar frames.
-- I'm not 100% certain exactly what all this does, but it should allow me to relocate where the bars end up.
-- The logic was copied from ShockAndAwe.
-----
function AweInspired:CreateBaseFrame()
	self.BaseFrame:SetScale(self.db.profile.scale);
	self.BaseFrame:SetFrameStrata("BACKGROUND");
	self.BaseFrame:SetWidth(300 + 175 / 7);
	self.BaseFrame:SetHeight(175);
	self.BaseFrame:SetBackdrop(self.frameBackdrop);
	self.BaseFrame:SetBackdropColor(1, 1, 1, 0);
	self.BaseFrame:SetMovable(true);
	self.BaseFrame:RegisterForDrag("LeftButton");
	self.BaseFrame:SetPoint(self.db.profile.point, self.db.profile.relativeTo, self.db.profile.relativePoint, self.db.profile.xOffset, self.db.profile.yOffset);
	self.BaseFrame:SetScript("OnDragStart",
		function()
			self.BaseFrame:StartMoving();
		end );
	self.BaseFrame:SetScript("OnDragStop",
		function()
			self.BaseFrame:StopMovingOrSizing();
			self.BaseFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.profile, self.BaseFrame);
		end ) ;
	self.BaseFrame:Show()
end


function AweInspired:FinishedMoving(var,frame)
	local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint();
	var.point = point
	var.relativeTo = relativeTo
	var.relativePoint = relativePoint
	var.xOffset = xOffset
	var.yOffset = yOffset
end


-----
-- Creates a new bar frame.  This will be located relative to the base frame and offset determined by
-- the value of the frameOffset variable.  This creates the statusbar and spark for the bar as well as
-- setting the overall size.
--
-- Parameters:
--   spellID: the spell ID of the frame to add.
--   frameColor: The RGB and transparency values for the color of the statusbar.
--   icon: boolean to determine if an Icon should be displayed for this bar.
-----
function AweInspired:CreateBarFrame(name, spellID, barType, frameColor, icon, frameLongBar)
	local spellName, _, spellIcon, _, _, _, _, _, _ = GetSpellInfo(spellID);

	--Create the frame and icon frame
	self.frames[name] = CreateFrame("Frame", spellID, self.BaseFrame);
	self.frames[name].icon = CreateFrame("Frame", spellID.."Icon", self.BaseFrame);

	--Store some frame specific variables
	self.frames[name].spellID = spellID;
	self.frames[name].spellName = spellName;
	self.frames[name].barType = barType;
	self.frames[name].longBar = frameLongBar;
	self.frames[name].lastVal = 0;
	self.frames[name].numReset = 0;
	if barType == "gcd" then
		self.frames[name].Width = (1.5/self.db.profile.baseCD) * self.db.profile.fWidth;
		self.frames[name].MaxVal = 1.5;
	else
		self.frames[name].Width = self.db.profile.fWidth;
		self.frames[name].MaxVal = self.db.profile.baseCD;
	end

	--Set the values of the base frame
	self.frames[name]:SetFrameStrata("LOW");
	self.frames[name]:SetWidth(self.frames[name].Width);
	self.frames[name]:SetHeight(self.db.profile.fHeight);
	self.frames[name]:ClearAllPoints();
	self.frames[name]:SetPoint("TOPLEFT", self.BaseFrame, "TOPLEFT", self.db.profile.fHeight, frameOffset * ((-1 * self.db.profile.fHeight) + 3));
	self.frames[name]:SetBackdrop(self.barBackdrop);
	self.frames[name]:SetBackdropColor(0,0,0,0.2);
	self.frames[name]:SetBackdropBorderColor(1,1,1,1);
	self.frames[name]:EnableMouse(false);

	--Create the status bar and set the values
	if not self.frames[name].statusbar then
		self.frames[name].statusbar = CreateFrame("StatusBar", nil, self.frames[name])
	end
	self.frames[name].statusbar:SetFrameLevel(1);
	self.frames[name].statusbar:ClearAllPoints();
	self.frames[name].statusbar:SetHeight(self.db.profile.fHeight - 6);
	self.frames[name].statusbar:SetWidth(self.frames[name].Width - 6);
	self.frames[name].statusbar:SetPoint("TOPLEFT", self.frames[name], "TOPLEFT", 3, -3);
	--self.frames[name].statusbar:SetStatusBarTexture("Interface/Tooltips/UI-Tooltip-Background");
	self.frames[name].statusbar:SetStatusBarTexture(media:Fetch("statusbar", self.db.profile.texture));
	self.frames[name].statusbar:SetStatusBarColor(frameColor.r, frameColor.g, frameColor.b, frameColor.a);
	self.frames[name].statusbar:SetMinMaxValues(0, self.frames[name].MaxVal);
	self.frames[name].statusbar:SetValue(self.db.profile.baseCD);

	--Create the spark and set the values
	if not self.frames[name].spark then
		self.frames[name].spark = self.frames[name].statusbar:CreateTexture(nil, "OVERLAY");
	end
	self.frames[name].spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark");
	self.frames[name].spark:SetWidth(16);
	self.frames[name].spark:SetHeight(self.db.profile.fHeight + 16);
	self.frames[name].spark:SetBlendMode("ADD");
	self.frames[name].spark:SetPoint("CENTER",self.frames[name].statusbar,"LEFT",(self.frames[name].Width - 6), 0);
	self.frames[name].spark:Show();

	--Set the OnUpdate event of this frame to call OnBarUpdate
	self.frames[name]:SetScript("OnUpdate",
		function()
			AweInspired:OnBarUpdate(self.frames[name]);
		end );

	--If we're supposed to display an icon, set the values for that
	if icon then
		self.frames[name].icon:SetWidth(self.db.profile.fHeight - 3)
		self.frames[name].icon:SetHeight(self.db.profile.fHeight - 3)
		self.frames[name].icon:SetPoint("TOPRIGHT", self.frames[name], "TOPLEFT", 0, -2)
		self.frames[name].icon:SetBackdrop({
			bgFile = spellIcon,
			--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = false, tileSize = 0, edgeSize = 12,
			insets = { left = 0, right = 0, top = 0, bottom = 0 } })
		self.frames[name].icon:Show();
	end

	self.frames[name]:Show();
	self.frames[name].spark:Show();
	frameOffset = frameOffset + 1;
end


-----
-- Called constantly from the OnUpdate event of each bar, this is where the bars actually decrease in value over time.
-- the barFrame parameter should contain the frame that triggered the event, decrement it's value.
-----
function AweInspired:OnBarUpdate(barFrame)
	local totCD, timeLeft, count, expirationTime;

	--self:Print("Adding bar of type: "..barFrame.barType);
	if barFrame.barType == "cooldown" or barFrame.barType == "gcd" then
		--self:Print("cooldown or gcd");
		timeLeft, totCD = self:GetCooldownValues(barFrame.spellID, barFrame.spellName, true, barFrame.MaxVal);
	elseif barFrame.barType == "buff" then
		timeLeft, totCD = self:GetBuffDebuffValues(true, barFrame.spellName);
	elseif barFrame.barType == "debuff" then
		timeLeft, totCD = self:GetBuffDebuffValues(false, barFrame.spellName);
	elseif barFrame.barType == "energy" then
		timeLeft, totCD = self:GetEnergyValues(barFrame.spellID);
	elseif barFrame.barType == "combo" then
		timeLeft, totCD = self:GetComboValues();
	end

	--If the cooldown is shorter than the bar, shrink it down a bit.
	if totCD > 2 and totCD < barFrame.MaxVal then
		barFrame.Width = (totCD / barFrame.MaxVal) * barFrame.Width;
		barFrame:SetWidth(barFrame.Width);
		barFrame.statusbar:SetWidth(barFrame.Width - 6);
		barFrame.statusbar:SetMinMaxValues(0, totCD);
		barFrame.MaxVal = totCD;
	end

	--If there is more time left on the CD than the max value of the status bar, check the longBar setting to see what to do
	if totCD > barFrame.MaxVal then
		if barFrame.longBar == "wait" then
			if timeLeft > barFrame.MaxVal then
				timeLeft = barFrame.MaxVal;
			end
		elseif barFrame.longBar == "scale" then
			timeLeft = (barFrame.MaxVal * timeLeft) / totCD;
		elseif barFrame.longBar == "long" then
			barFrame.Width = (totCD / barFrame.MaxVal) * barFrame.Width;
			barFrame:SetWidth(barFrame.Width);
			barFrame.statusbar:SetWidth(barFrame.Width - 6);
			barFrame.statusbar:SetMinMaxValues(0, totCD);
			barFrame.MaxVal = totCD;
		elseif barFrame.longBar == "hybrid" then
			if timeLeft > barFrame.MaxVal then
				timeLeft = (barFrame.MaxVal * timeLeft) / totCD;
			end
		else
			--This shouldn't ever happen, but just to prevent errors if it does, treat is like "wait"
			if timeLeft > barFrame.MaxVal then
				timeLeft = barFrame.MaxVal;
			end
		end
	end

	if timeLeft <= 0 and barFrame.lastVal - timeLeft > threshold and barFrame.numReset <= numReset then
		timeLeft = barFrame.lastVal
		barFrame.numReset = barFrame.numReset + 1;
	else
		barFrame.numReset = 0;
	end
	
	local sparkLoc = (timeLeft / barFrame.MaxVal) * (barFrame.Width - 6);

	if sparkLoc > 0 then
		barFrame.spark:Show();
	else
		barFrame.spark:Hide();
	end
	
	barFrame.lastVal = timeLeft;
	barFrame.statusbar:SetValue(timeLeft);
	barFrame.spark:SetPoint("CENTER", barFrame.statusbar, "LEFT", sparkLoc, -1);
end


function AweInspired:GetCooldownValues(spellID, spellName, extendExecute, maxVal)
	local cdStart, totCD, _ = GetSpellCooldown(spellID);
	local timeLeft = (cdStart + totCD) - GetTime();

	if extendExecute then
		local usable, nomana = IsUsableSpell(spellName);
		if not usable and not nomana then
			timeLeft = maxVal;
		end
	end
	return timeLeft, totCD;
end

function AweInspired:GetBuffDebuffValues(buff, spellName)
	local unitCode, unitFilter, timeLeft;
	if buff then
		unitCode = "player";
		unitFilter = "HELPFUL|PLAYER";
	else
		unitCode = "target";
		unitFilter = "HARMFUL|PLAYER";
	end
	local _, _, _, count, _, totCD, expirationTime, _, _, _, _ = UnitAura(unitCode, spellName, nil, unitFilter);

	if totCD == nil then
		timeLeft = 0;
		totCD = 0
	else
		timeLeft = expirationTime - GetTime();
	end

	return timeLeft, totCD;
end

function AweInspired:GetEnergyValues(spellID)
	local spellName, _, spellIcon, spellCost, _, spellPowerType, _, _, _ = GetSpellInfo(spellID);
	local energy = UnitMana("player");
	local powerType, _ = UnitPowerType("player");

	if powerType ~= 3 or spellPowerType ~= 3 then
		return 0, 0;
	end
	local timeLeft = (spellCost - energy) / 10.0;

	return timeLeft, (spellCost / 10.0);
end

function AweInspired:GetComboValues()
	local comboPoints = GetComboPoints("player", "target");
	return comboPoints * self.db.profile.baseCD / 5, self.db.profile.baseCD;
end

-----
-- This function is called with the /ais deleteframe command, it removes the frame for the given spell ID
-----
function AweInspired:DeleteFrame(info, spellID)
--	self:Print("delete frame: "..spellID);
--	local removed = false;
--	local offset = 0;
	if spellID == nil then
		spellID = 0;
	end
--	for index,value in pairs(self.frames) do
--		if removed then
--			value:SetPoint("TOPLEFT", self.BaseFrame, "TOPLEFT", self.db.profile.fHeight, offset * ((-1 * self.db.profile.fHeight) + 3));
--		end
--		self:Print("index = ["..index.."] and spellID = ["..spellID.."]");
--		if tostring(index) == tostring(spellID) then
--			self:Print("found a match!");
--			value:SetScript("OnUpdate", nil);
--			value.icon:Hide();
--			value:Hide();
			for i,v in pairs(self.db.profile.bars) do
				--self:Print("scanning saved bars v.spellID = ["..v.spellID.."] spellID = ["..spellID.."]");
				if tostring(v.spellID) == spellID then
					--self:Print("found saved bar");
					table.remove(self.db.profile.bars, i);
				end
			end
			self:RefreshConfig();
--			removed = true;
--		end
--		offset = offset + 1;
--	end
end


function AweInspired:ClearBars()
	wipe(self.db.profile.bars);
	self:RefreshConfig();
end



-----
-- This function handles all chat /commands, if no value is specified it opens the interface options for this mod, otherwise it runs the command.
-----
function AweInspired:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(AweInspired, "ais", "AweInspired", input);
	end
end

-----
-- This is called when selecting an new texture from the options menu, it updates the textures of each bar.
-----
function AweInspired:SetBarTexture(info, value)
	self.db.profile.texture = value;
	local barTexture = media:Fetch("statusbar", value);
	for i,v in pairs(self.frames) do
		v.statusbar:SetStatusBarTexture(barTexture);
	end
end

-----
-- This is called when selecting an new border from the options menu, it updates the border of each bar.
-----
function AweInspired:SetBorderTexture(info, value)
	self.db.profile.border = value;
	local newTexture = media:Fetch("border", value);
	if newTexture then
		self.barBackdrop.edgeFile = newTexture;
		for i,v in pairs(self.frames) do
			v:SetBackdrop(self.barBackdrop);
		end
		self:CreateBaseFrame();
	end
end

-----
-- This function is called whenever a profile changes, any changes to saved variables that need to take place immediately should be in here.
-----
function AweInspired:RefreshConfig()
	self:SetBorderTexture(_, self.db.profile.border);
	--self.barBackdrop.edgeFile = media:Fetch("border",self.db.profile.border);


	--This stops the scripts from running.
	for i,v in pairs(self.frames) do
		v:SetScript("OnUpdate", nil);
		v:Hide();
		v.icon:Hide();
		v.spark:Hide();
	end

	--Clear out all existing frames, reset frameOffset so the positioning starts back at the top.
	wipe(self.frames);
	frameOffset = 0;
	self:CreateBaseFrame();

	--Add any saved frames from the new profile.
	for i,v in pairs(self.db.profile.bars) do
		self:CreateBarFrame(v.name, v.spellID, v.barType, v.color, true, v.longBar)
	end
end
